#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>

#include "attiny3217_cnano.h"
#include "drv_usart.h"


#define CCP_SPM							0x9D
#define CCP_IOREG						0xD8

#define DATA_EEPROM_BASE_ADDR			0x1400
#define DATA_EEPROM_SIZE				256
#define DATA_EEPROM_PAGE_SIZE			64
#define DATA_EEPROM_NUM_PAGE			4

#define NVM_CMD_WRITE_PAGE				0x01
#define NVM_CMD_ERASE_PAGE				0x02
#define NVM_CMD_ERASE_AND_WRITE_PAGE	0x03
#define NVM_CMD_PAGEBUFFER_CLEAR		0x04
#define NVM_CMD_ERASE_EEPROM			0x06




void LED_Toggle(uint8_t pin)
{
	PORTA.OUTTGL = pin;
}




void LED_ON(uint8_t pin)
{
	PORTA.OUTSET = pin;
}




void LED_OFF(uint8_t pin)
{
	PORTA.OUTCLR = pin;
}




static void sysclk_init(void)
{
	CPU_CCP = 0xD8;
	CLKCTRL.MCLKCTRLA = CLKCTRL_CLKSEL_OSC20M_gc;

	CPU_CCP = 0xD8;

	/* Main Clock will pass through undivided (CLK_PER=CLK_MAIN),
	   regardless of the value of PDIV */
	CLKCTRL.MCLKCTRLB = 0x0;
}




static void board_init(void)
{
	/* LED0 - Negative logic operation */
	PORTA.DIRSET = PIN_LED0;
	PORTA.OUTSET = PIN_LED0;

	/* User Button - Negative logic operation */
	PORTB.DIRCLR = PIN_SW0;

	PORTB.PIN7CTRL = PORT_PULLUPEN_bm;
	PORTB.PIN7CTRL &= ~0x7;
}




int main(void)
{
	volatile uint8_t *peeprom_addr;
	uint32_t count;

	sysclk_init();

	board_init();

	// Initialization of peripherals
	USART_Printf_Init(115200);
	
	/* Needs delay to settle USART */
	_delay_ms(500);

	printf("ATtiny3217 CURIOSITY NANO - DATA EEPROM Test\r\n");

	printf("Write 256 bytes to the internal Data EEPROM\r\n");

	/* SPM(Self-Programming) is required before writing NVMCTRL.CTRLA */
	CPU_CCP = CCP_SPM;
	NVMCTRL.CTRLA = NVM_CMD_PAGEBUFFER_CLEAR;

	for(count = 0; count < 256; count++)
	{
		*(volatile uint8_t *)(DATA_EEPROM_BASE_ADDR + count) = count;

		/* SPM(Self-Programming) is required before writing NVMCTRL.CTRLA */
		CPU_CCP = CCP_SPM;
		NVMCTRL.CTRLA = NVM_CMD_ERASE_AND_WRITE_PAGE;

		while(NVMCTRL.STATUS & NVMCTRL_EEBUSY_bm);
	}

	printf("Read 256 bytes from the internal Data EEPROM\r\n");

	for(count = 0; count < DATA_EEPROM_SIZE; count++)
	{
		peeprom_addr = (volatile uint8_t *)(DATA_EEPROM_BASE_ADDR + count);

		if(count != *peeprom_addr)
		{
			printf("ERROR at %d --> 0x%X\r\n", count, *peeprom_addr);
			break;
		}
			
		printf("[%ld] 0x%X\r\n", count, *peeprom_addr);
	}

	if(count == DATA_EEPROM_SIZE)
		printf("Read done... verification done\r\n");

	while (1);
}

